// SimulTrough.h header file
//
// Copyright (C) 2007 Roberto Grena
//
//
// This file is part of SimulTrough(C) Library, version 1.0.
//
// SimulTrough Library is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// SimulTrough Library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (see the Appendix A of the User's Guide).
// If not, see <http://www.gnu.org/licenses/>.
//
//
// Last change: 24/11/2009

#ifndef RAY_SHOOT_H
#define RAY_SHOOT_H

#include<cmath>

#ifndef PI
#define PI 3.14159265358979323846
#endif

using namespace std;


class vt3 { 	// semplice classe per gestire vettori 3d 

    friend class absorb_tab;
    friend class opt_simulation;
    friend class ray;
    friend class surferror;
    friend class solar_rays;
      
	double x,y,z;
	vt3(double, double, double); // costruisce vettore dalle 3 componenti
	vt3(const vt3&); // copy constructor
	void assign(double,double,double); // assegna le componenti a un vetore gia' esistente 
	double normsq() const; // norma quadra
	double norm() const; // norma
	void rotate(const vt3& axis, double angle); // ruota il vettore intorno all'asse axis 
												// dell'angolo angle 
	
	vt3& operator = (const vt3& a) { // operatore di assegnazione	
		if (&a != this) {
			x=a.x;
			y=a.y;
			z=a.z;
		}
		return *this;
	}
	
	// varie operazioni: +, -, prod. per scalare * (dx e sx), divisione per scalare /, 
	// prod scalare *, prod. vettore ^
	friend const vt3
		operator + (vt3, vt3);
	friend const vt3
		operator - (vt3, vt3);
	friend const double
		operator * (vt3, vt3);
	friend const vt3
		operator * (double, vt3);
	friend const vt3
		operator * (vt3, double);
	friend const vt3
		operator / (vt3, double);
	friend const vt3
		operator ^ (vt3, vt3); 
};

inline void vt3::rotate (const vt3& axis, double angle) {
	vt3 axe = axis/axis.norm();
	vt3 p1 = axe*((*this)*axe);
	vt3 p2 = (*this)-p1;
	(*this) = p1 + cos(angle)*p2 + sin(angle)*(axis^p2);
}

inline vt3::vt3 (double xi, double yi, double zi) {
	x=xi;
	y=yi;
	z=zi;
}

inline vt3::vt3 (const vt3& a) {
	x=a.x;
	y=a.y;
	z=a.z;	
}

inline void vt3::assign (double xi, double yi, double zi) {
	x=xi;
	y=yi;
	z=zi;
}

inline double vt3::normsq () const {
	return x*x+y*y+z*z;
}

inline double vt3::norm () const {
	return sqrt(x*x+y*y+z*z);
}

inline const vt3 operator + (vt3 a, vt3 b) {
	return vt3(a.x+b.x, a.y+b.y, a.z+b.z);	
}

inline const vt3 operator - (vt3 a, vt3 b) {
	return vt3(a.x-b.x, a.y-b.y, a.z-b.z);	
}

inline const double operator * (vt3 a, vt3 b) {
	return a.x*b.x + a.y*b.y + a.z*b.z;
}

inline const vt3 operator * (double a, vt3 b) {
	return vt3(a*b.x, a*b.y, a*b.z);	
}

inline const vt3 operator * (vt3 a, double b) {
	return vt3(a.x*b, a.y*b, a.z*b);	
}

inline const vt3 operator / (vt3 a, double b) {
	return vt3(a.x/b, a.y/b, a.z/b);	
}

inline const vt3 operator ^ (vt3 a, vt3 b) {
	return vt3(a.y*b.z-a.z*b.y, a.z*b.x-a.x*b.z, a.x*b.y-a.y*b.x);	
}



inline const double pot (double a, int b) {
	for (int i=2;i<=b;i++) a*=a;
	return a;
}



typedef double (*reflect_s)(double cosinc, double freq);
typedef double (*refrac_index_function)(double freq);
typedef double (*absorption_function)(double length, double freq);


class absorb_tab { 	// tabella con 5 colonne: vettore del punto di impatto, energia,
					// eventuale errore

    friend class opt_simulation; 

	// qualche costante
    static const int initial_space = 30;
	static const int default_number_sector = 72;
	static const int increment = 10;

	// variabili di controllo:
	int number; // numero di righe occupate
	int max_n;	// numero massimo di righe
	bool fla; 	// dice se la somma e' stata eseguita 
	int num_sect; // numero settori in cui � diviso il tubo
	
	double min_distance;
	// spazio in cui viene allocata la tabella:
	double* storage;
	
	// spazio in cui viene allocata la somma sui settori
	double* sum_sector;
	double error; // errore
	// funzioni:
	void inflate(int); // aumenta lo spazio disponibile
	void makesum(); // esegue le somme parziali per settore
	
//public:
	absorb_tab(int sector = default_number_sector, int n = initial_space, double min_distance = 0); // costruisce tabella con n righe 
	absorb_tab(absorb_tab& a); // copy-constructor
	void add(const vt3& posiz, double energy, double errors); // aggiunge una riga
	vt3 fetch_pos(int n) const; //restituisce la posizione alla riga numero n
	double fetch_energy(int n) const; //restituisce l'energia alla riga numero n
	double fetch_error(int n) const; //restituisce l'errore alla riga numero n
	int get_number_lines() const;
	int get_max_lines() const;
	double get_total_error() const;
	
	int get_number_sector() const;
	void set_number_sector(int n); // modifica il numero di settori
	double energy_sector(int n); // energia nel settore n
	void clean(); // pulisce, ma non dealloca
	~absorb_tab(); // distruttore (dealloca)
	
	//const absorb_tab operator = (const absorb_tab& right); 
	
};


class ray { // contiene le informazioni su un raggio

	friend class opt_simulation;

//public:
		
	vt3 stp;
	vt3 direction;
	double energy;
	double frequency;
	
//public:
	
	// costruttori
	ray(const vt3& start, const vt3& direc, double energ, double freq);
	ray(double spx, double spy, double spz, double vx, double vy, double vz, 
			double energ, double freq);
	ray(const ray& cp); // copy-constructor
	
	// reflect: richiede vettore normale alla superficie, punto di impatto, 
	// e legge di riflessione (da passare come puntatore a funzione con 2 double
	// come argomenti); modifica il raggio riflettendolo
	void reflect(const vt3& normal_surface, const vt3& impact_point, 
					double (*reflect_law)(double cosinc, double freq) );
					
	// oltre a modificare il raggio riflettendolo, restituisce un puntatore al raggio
	// rifratto; chiede anche l'indice di rifrazione relativo
	ray* reflec_refrac(vt3 normal_surface, vt3 impact_point, double refr_index, 
						double (*reflect_law)(double cosinc, double freq) );
						
	//const ray operator = (const ray& right);
	
};


class surferror { // dati sugli errori della superficie parabolica e della posizione del tubo

//public:

    friend class opt_simulation;

	vt3 defoc;
	double gauss_trasv;
	int number_harm;
	double* coeff_harm;
	double gauss_long;
	
	double generate_gauss() const;
	
//public:
	
	surferror(const vt3& def = vt3(0,0,0), double gauss_tra = 0, int number_ha = 0, 
				double* coeff_ha = 0, double gauss_lo = 0 );
	surferror(const surferror& rg);
	
	void set_errors(const vt3& def = vt3(0,0,0), double gauss_tra = 0, int number_ha = 0, 
						double* coeff_ha = 0, double gauss_lo = 0 );
	void set_defoc(const vt3& def);
	void set_gauss_trasv(double gauss_tra);
	void set_coeff_harm(int number_ha, double* coeff_ha);
	void set_gauss_long(double gauss_lo);
	~surferror();
	
	//const surferror operator = (const surferror& right);
	
	void generate_error_surf(vt3& norm_surf, double xcoord) const;	
};


class solar_rays {

//public:
	
	friend class opt_simulation;
	
	static const double rad_disk = .004653;
	static const double rad_corona = .05585;
	static const double frac_corona = .1;
	static const int increm_ray = 20;
	
	int n_corone;
	int nray;
	int nray_disk;
	int max_ray;
	double energy_sun;
	
//public:
	
	double* data;
	
	static double solar_distrib(double r);
	void rotate_sun(vt3 axis, double ang);
	void posiz_sun(vt3 newpos);
	int get_nray() const;
	vt3 get_dir_sunray(int n) const;
	double get_energy_sunray(int n) const;
	
	solar_rays(int n);
//	solar_rays(const solar_rays& slr);
	//solar rays(int n, char);
	~solar_rays();
	
	//const 
    solar_rays& operator = (const solar_rays& right);
	
};



class opt_simulation {

//public:
	
	bool print_flag;
	
	double Fuoco; //= 1.81;			// focal distance
	double LarghSpecchi; //= 2.95;	// semi-width of the collector
	double Apertura; //= 0.025;		// gap between the two semi-parabolic mirrors
	int NumSurfaces; //= 3;			// number of concentric surfaces around the focal line
									// usually 3: glass extern surface - glass inner surface - cermet

	double *RaggioTubi; //= {0.0625, 0.0595, 0.035}; // radii of the concentric surfaces

	reflect_s reflect_mirror;
	reflect_s* reflect_surface; // 2*NumSurfaces-1
	refrac_index_function* refrac_index; // NumSurfaces
	absorption_function* extinction; // NumSurfaces

	
	int flag_freq;
	int n_sector;
	int n_step;
	double LimiteInsignificante; //= 1e-5; 	// lower limit on energy;
	int NRicMax; //= 30; 					// maximum number of recursion allowed;
	double StepStep; //= 1e-6; 				// shift of the rays starting from surfaces (lower
											// limit on the dimension of objects)
	absorb_tab cermet_abs;
	absorb_tab glass_abs;
	
	double errore;
	double* glass;
	double* cermet;
	double efficiency;
	
	opt_simulation(const opt_simulation&);
	opt_simulation operator = (const opt_simulation&);
	
	double* tabella_lambda;
	
	surferror errori;
	solar_rays slr;
	
	ray current_ray;

	void effext(ray* shot, int n_ric);
	void effcirc(ray* shot, int number_surf, int n_ric);
	
	void shoot(const vt3& start, const vt3& direc, double energ, double freq);
	

public:
	
    opt_simulation(

                                   // geometric parameters
                                   double Focus, 
                                   double MirrorWidth, 
                                   double FreeSpace,
                                   int NumSurface,
                                   double* RadiiReceiver,

                                   // optical functions
                                   reflect_s ReflectMirror,
                                   reflect_s* ReflectSurf,
                                   refrac_index_function* RefracIndex,
                                   absorption_function* AbsorptionFunctions,

                                   // simulation technical parameters
                                   int nAnnuli = 10,
                                   int nSector = 72,
                                   int nStep = 200,
                                   int nLambda = 1,
                                   double LimitEnergy = 1e-5,
                                   int NRicLim = 30,
                                   double Offset = 1e-5,

                                   // errori
                                   double Defocalization_x = 0,
                                   double Defocalization_y = 0,
                                   double GaussTrasv = 0,
                                   double GaussLong = 0,
                                   int NumberHarm = 0,
                                   double* CoeffHarm = 0	 );
		
	~opt_simulation();
					
	void simulate(double sunpos_x, double sunpos_y, double sunpos_z, 
         double shadow_sx = 0, double shadow_dx = 0);	
	
    void printoff();
    void printon();
	
	double get_absorber(int NumberSector) const;
	double get_absorb_glass(int NumberSector) const;
	double get_efficiency() const;
	double get_rec_error() const;
	
	double get_Focus() const;
	double get_MirrorWidth() const;
	double get_NumSurface() const;
	double get_RadiiReceiver(int number_surface) const;
	double get_FreeSpace() const;

    void set_Focus(double newFocus);
    void set_MirrorWidth(double newMirrorWidth);
    void set_RadiiReceiver(int NumberSurface, double newRadius);
    void set_FreeSpace(double newFreeSpace);
    void set_geometry(double newFocus, double newMirrorWidth, 
         double newFreeSpace, double* newRadiiReceiver);

    void set_NumSurface(int newNumSurface, double* newRadiiReceiver,
         reflect_s* list_newReflectSurf, refrac_index_function* list_newRefInd, 
         absorption_function* list_newAbsorption);
	
    void set_ReflectMirror(reflect_s newReflectMir);
    void set_ReflectSurfaces(int NumberFun, reflect_s
	     newReflectSurface);
    void set_RefracIndex(int NumberFun,
         refrac_index_function newRefIndex);
    void set_Absorption(int NumberFun, 
         absorption_function newAbsorption);

	int get_nAnnuli() const;
	int get_nSolarRay() const;
	int get_nSector() const;
	int get_nStep() const;
	double get_LimitEnergy() const;
	int get_NRicLim() const;
	double get_Offset() const;
	int get_nLambda() const;

    void set_nAnnuli(int new_nAnnuli);
    void set_nSector(int new_nSector);
    void set_nStep(int new_nStep);
    void set_LimitEnergy(double new_LimitEnergy);
    void set_NRicLim(int new_NRicLim);
    void set_Offset(int new_Offset);
    void set_nLambda(int new_nLambda);

    void set_param_simulation(int new_nAnnuli = 10,
			int new_nSector = 72,
			int new_nStep = 200,
            int new_nLambda = 1,
			double new_LimitEnergy = 1e-5,
			int new_NRicLim = 30,
			double new_Offset = 1e-5);
	
	double get_ErrorDefoc_x() const;
	double get_ErrorDefoc_y() const;
	double get_ErrorGaussTrasv() const;
	double get_ErrorGaussLong() const;
	int get_ErrorNumberHarm() const;
	double get_ErrorHarm(int indexHarm) const;
	
    void set_ErrorDefoc(double ErrorDefoc_x,
         double ErrorDefoc_y);
    void set_ErrorGaussTrasv(double sigma);
    void set_ErrorGaussLong(double sigma);
    void set_ErrorTrasv(int NumberHarm, double* CoeffHarm);
    void set_Errors(double ErrorDefoc_x, double ErrorDefoc_y, 
         double ErrorGaussTrasv, double ErrorGaussLong, 
         int NumberHarm, double* CoeffHarm);
	
};

#endif
