// header file opt_fun_definition.h
//
// Copyright (C) 2007 Roberto Grena
//
//
// This file is part of SimulTrough(C) Library, version 1.0.
//
// SimulTrough Library is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// SimulTrough Library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. (see the Appendix A of the User's Guide).
// If not, see <http://www.gnu.org/licenses/>.


#ifndef OPT_FUN_DEFINITION
#define OPT_FUN_DEFINITION


// DEFINITIONS OF THE REFLECTIVITY FUNCTIONS //

// the reflectivity can depend from the angle of incidence om the surface and 
// from the wavelength of the light. In the following functions, the variable 
// cosinc is the cosine of the incidence angle on the surface, and lambda is 
// the wavelength of the incident light

// MirrorReflectivity: reflectivity function of the mirror

double MirrorReflectivity (double cosinc, double lambda) {
       return 1-0.1*cosinc;
}


// ReflectGlassExternOut: reflectivity of the external face of the glass, 
// when the light comes from the outside (from the air)

double ReflectGlassExternOut (double cosinc, double lambda) {

       double R_perp = (cosinc-sqrt(1.25+cosinc*cosinc))/(cosinc+
              sqrt(1.25+cosinc*cosinc));
       
       double R_par = (2.25*cosinc-sqrt(1.25+cosinc*cosinc))/(2.25*cosinc+
              sqrt(1.25+cosinc*cosinc));
       
       return 0.5*(R_perp*R_perp+R_par*R_par);	
}


// ReflectGlassExternIn: reflectivity of the external face of the glass, 
// when the light comes from the inside (from the glass)

double ReflectGlassExternIn (double cosinc, double lambda) {

	if ( cosinc*cosinc <= 0.5556)  return 1;    // total reflection
    double R_perp = (cosinc-sqrt(-0.5556+cosinc*cosinc))/(cosinc
              +sqrt(-0.5556+cosinc*cosinc));
    
    double R_par = (cosinc-1.5*sqrt(-1.25+2.25*cosinc*cosinc))/(cosinc+
              1.5*sqrt(-1.25+2.25*cosinc*cosinc));

	return 0.5*(R_perp*R_perp + R_par*R_par);	
}



// ReflectGlassInternOut: reflectivity of the internal face of the glass, 
// when the light comes from the "outside" (from the glass)

double ReflectGlassInternOut (double cosinc, double lambda) {

	if ( cosinc*cosinc <= 0.5556)  return 1;    // total reflection

    double R_perp = (cosinc-sqrt(-0.5556+cosinc*cosinc))/(cosinc
           +sqrt(-0.5556+cosinc*cosinc));

    double R_par = (cosinc-1.5*sqrt(-1.25+2.25*cosinc*cosinc))/(cosinc+
           1.5*sqrt(-1.25+2.25*cosinc*cosinc));

	return 0.5*(R_perp*R_perp + R_par*R_par);	
}



// ReflectGlassInternIn: reflectivity of the internal face of the glass, 
// when the light comes from the "inside" (from the vacuum)

double ReflectGlassInternIn (double cosinc, double lambda) {

       double R_perp = (cosinc-sqrt(1.25+cosinc*cosinc))/(cosinc+
              sqrt(1.25+cosinc*cosinc));
       
       double R_par = (2.25*cosinc-sqrt(1.25+cosinc*cosinc))/(2.25*cosinc+
              sqrt(1.25+cosinc*cosinc));
       
       return 0.5*(R_perp*R_perp+R_par*R_par);	
}


// ReflectAbsorber: reflectivity of the absorbing surface

double ReflectAbsorber (double cosinc, double lambda) { 
       return 1-0.95*cosinc; 
}



// DEFINITIONS OF REFRACTIVE INDEX FUNCTIONS //

// in the refractive index functions, the only dependence is from the 
// wavelength of th elight. The parameter lambda is the wavelength 

// RefIndex_air: refractive index of the outer space (the air)
double RefIndex_air (double lambda) { return 1.0003; }

// RefIndex_glass: refractive index of the glass
double RefIndex_glass (double lambda) { return 1.5; }

// RefIndex_vacuum: refractive index of the vacuum
double RefIndex_vacuum(double lambda) { return 1; }



// DEFINITIONS OF THE ABSORPTION FUNCTIONS //

// the absorption can depend on the length of the travel in the medium and on 
// the wavelength of the light (respectively called length and lambda in the 
// following functions). The function must return the ratio between the final 
// energy of the ray after passing in the medium and the initial energy of 
// the ray ( and _not_ the fraction of energy absorbed).


// Absorption_air: absorption function of the air
double Absorption_air(double length, double lambda) { return 1; }

// Absorption_glass: absorption function of the glass
double Absorption_glass(double length, double lambda) 
       { return exp(-3*length); }

// Absorption_vacuum: absorption function of the vacuum
double Absorption_vacuum(double length, double lambda) { return 1; }


// definition of OPT_FUN_INIT, the macro that assigns the optical functions 
// to the pointers 

// the types reflect_s and absorption_function are pointers to function 
// double(double,double)
// the type refrac_index_function is a pointer to a function double(double)


#define OPT_FUN_INIT \
\
reflect_s ReflectMir = &MirrorReflectivity; \
\
\
reflect_s ReflectSurfaces[5]; \
ReflectSurfaces[0] = &ReflectGlassExternOut;\
ReflectSurfaces[1] = &ReflectGlassExternIn;\
ReflectSurfaces[2] = &ReflectGlassInternOut;\
ReflectSurfaces[3] = &ReflectGlassInternIn;\
ReflectSurfaces[4] = &ReflectAbsorber;\
\
refrac_index_function RefInd[3];\
RefInd[0]=&RefIndex_air;\
RefInd[1]=&RefIndex_glass;\
RefInd[2]=&RefIndex_vacuum;\
\
absorption_function Absorption[3];\
Absorption[0]=&Absorption_air;\
Absorption[1]=&Absorption_glass;\
Absorption[2]=&Absorption_vacuum


#endif

// end of file
